#include "StdAfx.h"
#include ".\roicoder.h"
#include "..\CVCommon.h"

ROICoder::ROICoder(int width, int height, bool circular):mem(NULL)
{
	this->circular = circular;
	curstate = BEGIN;
	regionmask= cvCreateImage( cvSize(width,height), IPL_DEPTH_8S, 1);
	for (int i=0; i<width; i++)
		for (int j=0; j<height; j++){
			SetSingleIntValue(regionmask,i,j,1);
		}

	cur_node = NULL;
	rowmin = width+1;
	rowmax = 0;
	colmin = height +1;
	colmax = 0;
	contour_num = 0;
}

ROICoder::~ROICoder(void)
{
	cvReleaseMemStorage( &mem );
//	if(contour)
//		cvClearSeq( contour );
	cvReleaseImage(&regionmask);
}

void ROICoder::BeginContour(CvPoint startpnt)
{
	if( contour_num >= 9)
		return;
	this->startpnt = startpnt;
	mem = cvCreateMemStorage(0);
	contour[contour_num] = cvCreateSeq(0,sizeof(CvSeq),sizeof(ImNode),mem);//CV_SEQ_ELTYPE_POINT
	ImNode node;
	node.pos.x = startpnt.x;
	node.pos.y = startpnt.y;
	cur_node = (ImNode *)cvSeqPush(contour[contour_num],&node);//The first relative positioncvPoint(startpnt.x,startpnt.y)
	SetSingleIntValue(regionmask,startpnt.x,startpnt.y,0);
	prepnt = startpnt;
	curstate = PROCEED;

	rowmin = imageWidth() +1;
	rowmax = 0;
	colmin = imageHeight() +1;
	colmax = 0;
}

void ROICoder::AddContourNode(CvPoint pnt)
{	
	if((curstate != PROCEED) || (pnt.x==prepnt.x && pnt.y==prepnt.y))
		return;
	
	{//////////Bresenham Algorithm Implemented here/////////////
		int x=pnt.x;
		int y=pnt.y;
		int signs=1;
		if(x==prepnt.x)
			signs=0;
		int x0=prepnt.x,y0=prepnt.y;
		int dx=abs(x-x0),dy=abs(y-y0);
		int s1=trisign(x-x0),s2=trisign(y-y0);
		int tmp;
		int Interchange;
		if(dy>dx){
			tmp=dx;
			dx=dy;
			dy=tmp;
			Interchange=1;
		}else{
			Interchange=0;
		}
		int e=2*dy-dx;

		for(int i=1;i<=dx;i++){//(x0,y0)is not included	
			while(e>0){
				if(Interchange==1)
					x0+=s1;
				else
					y0+=s2;
				e=e-2*dx;
			}
			if(Interchange==1)
				y0+=s2;
			else
				x0+=s1;
			e=e+2*dy;
			ImNode node;
			node.pos.x = x0;
			node.pos.y = y0;
			if(x0 < rowmin){
				rowmin = x0;
			}
			if(x0 > rowmax){
				rowmax = x0;
			}
			if(y0 < colmin) {
				colmin = y0;
			}
			if(y0 > colmax) {
				colmax = y0;
			}
////Begin Encode the node type of the previous node, cur_node should already been initialized
			int dirx=x0-cur_node->pos.x,diry=y0-cur_node->pos.y;
			ASSERT((dirx!=0 || diry!=0));
			switch (dirx){				
				case 1:
					if(diry==0)
						cur_node->dir=0;
					else if(diry==-1)
						cur_node->dir=1;
					else if(diry==1)
						cur_node->dir=7;
					else
						goto errors;
					break;
				case -1:
					if(diry==0)
						cur_node->dir=4;
					else if(diry==-1)
						cur_node->dir=3;
					else if(diry==1)
						cur_node->dir=5;
					else
						goto errors;
					break;
				case 0:
					if(diry==-1)
						cur_node->dir=2;
					else if(diry==1)
						cur_node->dir=6;
					else
						goto errors;
					break;
				default:
errors:	//Seems No Problem	
					ASSERT(false);
			}//move on
//////End of Encoding the node type of the previous node
			cur_node = (ImNode *)cvSeqPush(contour[contour_num], &node);//Add relative position of x0 y0 to chain cvPoint(x0,y0)
			SetSingleIntValue(regionmask,x0,y0,0);//set region map
		}	
	}
	prepnt = pnt;
}

void ROICoder::EndContour()
{
	AddContourNode(startpnt);
	curstate = BEGIN;
	CodeNType();
	CodeRegion();
	contour_num ++;
}

CvSeq * ROICoder::getContour(int i)
{
	if(i < contour_num)
		return contour[i];
	else 
		return NULL;
}

int ROICoder::imageHeight()
{
	return  regionmask->height;
}

int ROICoder::imageWidth()
{
	return  regionmask->width;
}
void ROICoder::CodeNType(void)
{
	if(curstate != END)
		return;
	CvSeq *seq = contour[contour_num];
	ImNode *prev_node, *cur_node;
	prev_node = (ImNode*)cvGetSeqElem(seq, 0);
	for (int i=1; i<seq->total; i++){
		cur_node = (ImNode*)cvGetSeqElem(seq, i);
		int x=prev_node->dir,y=cur_node->dir;
		switch(x){
		case 0:
			if(y==2||y==3)
				cur_node->ntype= ImNode::S;
			if(y==5)
				cur_node->ntype= ImNode::M;
			break;
		case 1:
			if(y==2||y==3||y==4)
				cur_node->ntype= ImNode::S;
			break;
		case 3:
			if(y==0||y==1||y==2)
				cur_node->ntype= ImNode::M;
			break;
		case 4:
			if(y==1||y==2)
				cur_node->ntype= ImNode::M;
			if(y==7)
				cur_node->ntype= ImNode::S;
			break;
		case 5:
			if(y==7||y==0)
				cur_node->ntype= ImNode::S;
			break;
		case 6:
			if(y==5||y==4)
				cur_node->ntype= ImNode::M;
			if(y==0||y==7)
				cur_node->ntype= ImNode::S;
			break;
		case 7:
			if(y==5||y==4)
				cur_node->ntype=ImNode::M;
			break;
		default:
			break;
		}// End of switch

		prev_node = cur_node;
	}
}

// Encode the inner region of the contour, denoted -1 in regionmask
void ROICoder::CodeRegion(void)
{
	int i=rowmin+1;
	const int H=imageHeight();		
	while(i<rowmax){
		int isfirst=1;
		int fpart=1;
		int j=colmin;
		/////////////////////Each Row/////////////////////
		while(j<colmax){
			while(j<colmax){
				if(GetSingleIntValue(regionmask, i,j) != 0){
					if(!fpart){
						SetSingleIntValue(regionmask, i,j, -1);
					}
					j++;
				}
				else
					break;
			}
			if(fpart){
				while((j+1)<colmax){
					if(GetSingleIntValue(regionmask, i,j+1)==0)
						j++;
					else
						break;
				}
			}
			if((j+1)>=H)
				break;
			//////////////////find an edge node////////////////////
			if(fpart){//First [a,..]
				if(isfirst){
					if(FindNodeType(cvPoint(i,j))==ImNode::M){//No action
						j++;
						continue;
					}										
					SetSingleIntValue(regionmask, i,j+1, -1);
					isfirst=0;
				}else{
					if(FindNodeType(cvPoint(i,j))==ImNode::M){//No action
						j++;
						continue;
					}					
					SetSingleIntValue(regionmask, i,j+1, -1);					
				}
				fpart=0;
			}else{//Second [.., p]
				if(FindNodeType(cvPoint(i,j))==ImNode::S){//Split node
					SetSingleIntValue(regionmask, i,j-1, -1);					
					j++;
					continue;
				}
				SetSingleIntValue(regionmask, i,j-1, -1);		
				fpart=1;
			}
			//////////////////////////////////////////////
			j++;
		}
		//////////////////////////////////////////////////////
		i++;
	}
}

// Find the node type in the position pos
ImNode::NodeType ROICoder::FindNodeType(CvPoint pos)
{
	CvSeq *seq = contour[contour_num];
	for(int i=0; i<seq->total; i++){
		ImNode *node =  (ImNode*)cvGetSeqElem(seq, i);
		if((node->pos.x == pos.x) && (node->pos.y == pos.y)){
			return node->ntype;
		}
	}
	ASSERT(false);
	return ImNode::M;
}

IplImage * ROICoder::GetRegionMask(void)
{
	return regionmask;
}

int ROICoder::getCNum()
{
	return contour_num;
}


