/* graph.cpp */
/* Vladimir Kolmogorov (vnk@cs.cornell.edu), 2001. */
/* Modified Version by ...*/

#include <stdio.h>
#include ".\graph.h"


Graph::Graph(ErrorFunction err_function) : 
NODE_BLOCK_SIZE(512),
ARC_BLOCK_SIZE(1024),
NODEPTR_BLOCK_SIZE(128),
INFINITE_D(10000000),
STATE_FREE(FREE)
{
	error_function = err_function;
	node_block = new Block<node>(NODE_BLOCK_SIZE, error_function);
	arc_block  = new Block<arc>(NODE_BLOCK_SIZE, error_function);
	flow = 0;
}

Graph::~Graph(void)
{
	delete node_block;
	delete arc_block;
}

Graph::node_id Graph::add_node()
{
	node *i = node_block -> New();

	i -> tr_cap = 0;
	i -> tree_type = NONE;
	i -> node_state = &STATE_FREE;

	return (node_id) i;
}

void Graph::set_tweights(node_id i, CapType cap_source, CapType cap_sink)
{
	flow += (FlowType)((cap_source < cap_sink) ? cap_source : cap_sink);
	((node*)i) -> tr_cap = cap_source - cap_sink;
}

void Graph::add_tweights(node_id i, CapType cap_source, CapType cap_sink)
{
	register CapType delta = ((node*)i) -> tr_cap;
	if (delta > 0) cap_source += delta;
	else           cap_sink   -= delta;
	flow += (FlowType)((cap_source < cap_sink) ? cap_source : cap_sink);
	((node*)i) -> tr_cap = cap_source - cap_sink;
}
