/*
	usage:
	{
		SDistMap dmap;
		dmap.begin_contour();

		dmap.add_point(i, j);
		...

		dmap.end_contour()

		dmap.code_DMap();
        
	}

*/

#pragma once

#include <List>
#include <vector>
#include <queue>
#include <iostream>

#include <crtdbg.h>

using namespace std;

class SDistMap
{
/////////////////////////////Interface//////////////////////////////////
public:
	typedef void (*ErrorFunction)(char* errmsg);
	enum Label {INNER, EDGE, OUTER};

public:
	void begin_contour(int x, int y);
	void end_contour();
	void add_point(int x, int y);

	void code_DMap();		//must be called outside the begin-end pair

	Label what_Label(int x, int y);
	double what_Distance(int x, int y);
	double what_SDistance(int x, int y);

	// Indicate whether being in between of  begin_contour() and end_contour()
	bool is_proceeding();

public:
	SDistMap(int width, int height, ErrorFunction errorFun = NULL);
	~SDistMap(void);








///////////////////////////Implementation///////////////////////////////

//////// constants and structures ////////
private:
	struct point
	{
		int x, y;
		point(){ x = 0; y = 0; }
		point(int ix, int iy){ x = ix; y = iy; }				
	};

	struct contour_list
	{
       vector<point> pnts;		
		point m_startPnt;
		point m_prevPnt;			

		//Help to reduce the number of iterations in label coding phase
		int m_numRMax, m_numRMin, m_numCMax, m_numCMin;	

		contour_list *next;				
	};

	typedef contour_list* contour_list_ptr;

	const int m_width, m_height;	
	const double INFINITE_D;

//////// Variables ////////
private:
	enum State {TOBEGIN, PROCEED} m_curStat;	//	

	ErrorFunction errfunction;
	
   double *m_DMap;		//Distance map
   Label		*m_LMap;	//Label map

   list<point> m_actSet;
   contour_list_ptr m_contourFirst;
   contour_list_ptr m_contourLast;	//Last and current

//////// Operations ////////
private:
	// Some utility functions
	inline double getDMap(int x, int y) {	return m_DMap[x*m_height + y]; }
	inline void setDMap(int x, int y, double val) {	 m_DMap[x*m_height + y] = val; }
	inline Label getLMap(int x, int y) { return m_LMap[x*m_height + y]; }
	inline void setLMap(int x, int y, Label lb) { m_LMap[x*m_height + y] = lb; }
	inline double getSDMap(int x, int y) {	
		if(what_Label(x, y) == OUTER) {
			return m_DMap[x*m_height + y]; 
		}else {
			return -m_DMap[x*m_height + y];
		}
	}

	inline int tri_sign(int x) { return (x>0) ? 1 : ((x<0) ? -1: 0); }
	
	inline point extract_front() {
		point res = m_actSet.front();
		m_actSet.pop_front();
		return res;
	}

	inline void saturate(int &x, int &y) {
		if( x == 0 ) x = 1;
		if( y == 0 ) y = 1;
		if( x == m_width - 1) x = m_width -2;
		if( y == m_height - 1) y = m_height - 2;
	}

//////// Main Functions //////////
private:
	// Get the index of the point (x, y) in the last contour
	int getInvPointIndex(int x, int y);

	// Initialize active set and distance map
	void initDMap();
	
	// Infer the INNER label based on the LMap, only edge point is set initially, O(N) complexity	
	void codeLMap();

	// Calculation of the distance using BFS, O(N) complexity
	void codeDMap();
};
